Instructions to use replicate/flashinfer-draft with libraries, inference providers, notebooks, and local apps. Follow these links to get started.
- Libraries
- Kernels
How to use replicate/flashinfer-draft with Kernels:
# !pip install kernels from kernels import get_kernel kernel = get_kernel("replicate/flashinfer-draft") - Notebooks
- Google Colab
- Kaggle
File size: 8,705 Bytes
57c3a10 | 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 | /*
* Copyright (c) 2024 by FlashInfer team.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
#ifndef FLASHINFER_DECODE_PARAMS_CUH_
#define FLASHINFER_DECODE_PARAMS_CUH_
#include <cuda_runtime.h>
#include <cstdint>
#include "../layout.cuh"
#include "../page.cuh"
namespace flashinfer {
template <typename DTypeQ_, typename DTypeKV_, typename DTypeO_>
struct SingleDecodeParams {
using DTypeQ = DTypeQ_;
using DTypeKV = DTypeKV_;
using DTypeO = DTypeO_;
using IdType = int32_t;
DTypeQ* q;
DTypeKV* k;
DTypeKV* v;
DTypeO* o;
float* lse;
float* maybe_alibi_slopes;
uint32_t kv_len;
uint32_t num_qo_heads;
uint32_t num_kv_heads;
uint32_t q_stride_n;
uint32_t q_stride_h;
uint32_t kv_stride_n;
uint32_t kv_stride_h;
int32_t window_left;
float logits_soft_cap;
float sm_scale;
float rope_rcp_scale;
float rope_rcp_theta;
uint32_t kv_chunk_size;
__device__ __host__ SingleDecodeParams()
: q(nullptr),
k(nullptr),
v(nullptr),
o(nullptr),
lse(nullptr),
maybe_alibi_slopes(nullptr),
kv_len(0),
num_qo_heads(0),
num_kv_heads(0),
q_stride_n(0),
q_stride_h(0),
kv_stride_n(0),
kv_stride_h(0),
window_left(0),
logits_soft_cap(0.0f),
sm_scale(0.0f),
rope_rcp_scale(0.0f),
rope_rcp_theta(0.0f),
kv_chunk_size(0) {}
__device__ __host__ SingleDecodeParams(DTypeQ* q, DTypeKV* k, DTypeKV* v, DTypeO* o,
float* maybe_alibi_slopes, uint32_t seq_len,
uint32_t num_qo_heads, uint32_t num_kv_heads,
QKVLayout kv_layout, uint32_t head_dim,
int32_t window_left, float logits_soft_cap, float sm_scale,
float rope_scale, float rope_theta)
: q(q),
k(k),
v(v),
o(o),
lse(nullptr),
maybe_alibi_slopes(maybe_alibi_slopes),
kv_len(seq_len),
num_qo_heads(num_qo_heads),
num_kv_heads(num_kv_heads),
q_stride_n(num_qo_heads * head_dim),
q_stride_h(head_dim),
kv_stride_n((kv_layout == QKVLayout::kNHD) ? num_kv_heads * head_dim : head_dim),
kv_stride_h((kv_layout == QKVLayout::kNHD) ? head_dim : seq_len * head_dim),
window_left(window_left),
logits_soft_cap(logits_soft_cap),
sm_scale(sm_scale),
rope_rcp_scale(1.f / rope_scale),
rope_rcp_theta(1.f / rope_theta),
kv_chunk_size(0) {}
__host__ __device__ __forceinline__ uint32_t get_qo_len(uint32_t batch_idx) const { return 1; }
__host__ __device__ __forceinline__ uint32_t get_kv_len(uint32_t batch_idx) const {
return kv_len;
}
};
template <typename DTypeQ_, typename DTypeKV_, typename DTypeO_, typename IdType_>
struct BatchDecodeParams {
using DTypeQ = DTypeQ_;
using DTypeKV = DTypeKV_;
using DTypeO = DTypeO_;
using IdType = IdType_;
DTypeQ* q;
IdType* q_rope_offset;
paged_kv_t<DTypeKV, IdType> paged_kv;
DTypeO* o;
float* lse;
float* maybe_alibi_slopes;
uint32_t padded_batch_size;
uint32_t num_qo_heads;
IdType q_stride_n;
IdType q_stride_h;
int32_t window_left;
float logits_soft_cap;
float sm_scale;
float rope_rcp_scale;
float rope_rcp_theta;
IdType* request_indices;
IdType* kv_tile_indices;
IdType* o_indptr;
IdType* kv_chunk_size_ptr;
bool* block_valid_mask;
bool partition_kv;
__device__ __host__ BatchDecodeParams()
: q(nullptr),
q_rope_offset(nullptr),
paged_kv(),
o(nullptr),
lse(nullptr),
maybe_alibi_slopes(nullptr),
padded_batch_size(0),
num_qo_heads(0),
q_stride_n(0),
q_stride_h(0),
window_left(0),
logits_soft_cap(0.0f),
sm_scale(0.0f),
rope_rcp_scale(0.0f),
rope_rcp_theta(0.0f),
request_indices(nullptr),
kv_tile_indices(nullptr),
o_indptr(nullptr),
kv_chunk_size_ptr(nullptr),
block_valid_mask(nullptr),
partition_kv(false) {}
__device__ __host__ BatchDecodeParams(DTypeQ* q, IdType* q_rope_offset,
paged_kv_t<DTypeKV, IdType> paged_kv, DTypeO* o, float* lse,
float* maybe_alibi_slopes, uint32_t num_qo_heads,
IdType q_stride_n, IdType q_stride_h, int32_t window_left,
float logits_soft_cap, float sm_scale, float rope_scale,
float rope_theta)
: q(q),
q_rope_offset(q_rope_offset),
paged_kv(paged_kv),
o(o),
lse(lse),
maybe_alibi_slopes(maybe_alibi_slopes),
padded_batch_size(0),
num_qo_heads(num_qo_heads),
q_stride_n(q_stride_n),
q_stride_h(q_stride_h),
window_left(window_left),
logits_soft_cap(logits_soft_cap),
sm_scale(sm_scale),
rope_rcp_scale(1.f / rope_scale),
rope_rcp_theta(1.f / rope_theta),
request_indices(nullptr),
kv_tile_indices(nullptr),
o_indptr(nullptr),
kv_chunk_size_ptr(nullptr),
block_valid_mask(nullptr),
partition_kv(false) {}
__host__ __device__ __forceinline__ int32_t get_qo_len(int32_t batch_idx) const { return 1; }
__host__ __device__ __forceinline__ int32_t get_kv_len(int32_t batch_idx) const {
return paged_kv.get_length(batch_idx);
}
};
template <typename DTypeQ_, typename DTypeKV_, typename DTypeO_, typename IdType_>
struct BatchDecodeParamsMLA {
using DTypeQ = DTypeQ_;
using DTypeKV = DTypeKV_;
using DTypeO = DTypeO_;
using IdType = IdType_;
DTypeQ* q_nope;
DTypeQ* q_pe;
DTypeO* o;
float* lse;
float sm_scale;
IdType* q_rope_offset;
paged_kv_mla_t<DTypeKV, IdType> paged_kv;
uint32_t padded_batch_size;
uint32_t num_qo_heads;
int32_t window_left;
float logits_soft_cap;
float rope_rcp_scale;
float rope_rcp_theta;
IdType* request_indices;
IdType* kv_tile_indices;
IdType* o_indptr;
IdType* kv_chunk_size_ptr;
bool* block_valid_mask;
bool partition_kv;
__device__ __host__ BatchDecodeParamsMLA()
: q_nope(nullptr),
q_pe(nullptr),
o(nullptr),
lse(nullptr),
sm_scale(0.0f),
q_rope_offset(nullptr),
paged_kv(),
padded_batch_size(0),
num_qo_heads(0),
window_left(0),
logits_soft_cap(0.0f),
rope_rcp_scale(0.0f),
rope_rcp_theta(0.0f),
request_indices(nullptr),
kv_tile_indices(nullptr),
o_indptr(nullptr),
kv_chunk_size_ptr(nullptr),
block_valid_mask(nullptr),
partition_kv(false) {}
__device__ __host__ BatchDecodeParamsMLA(DTypeQ* q_nope, DTypeQ* q_pe, IdType* q_rope_offset,
paged_kv_mla_t<DTypeKV, IdType> paged_kv, DTypeO* o,
float* lse, uint32_t num_qo_heads, int32_t window_left,
float logits_soft_cap, float sm_scale, float rope_scale,
float rope_theta)
: q_nope(q_nope),
q_pe(q_pe),
o(o),
lse(lse),
sm_scale(sm_scale),
q_rope_offset(q_rope_offset),
paged_kv(paged_kv),
padded_batch_size(0),
num_qo_heads(num_qo_heads),
window_left(window_left),
logits_soft_cap(logits_soft_cap),
rope_rcp_scale(1.f / rope_scale),
rope_rcp_theta(1.f / rope_theta),
request_indices(nullptr),
kv_tile_indices(nullptr),
o_indptr(nullptr),
kv_chunk_size_ptr(nullptr),
block_valid_mask(nullptr),
partition_kv(false) {}
__host__ __device__ __forceinline__ int32_t get_qo_len(int32_t batch_idx) const { return 1; }
__host__ __device__ __forceinline__ int32_t get_kv_len(int32_t batch_idx) const {
return paged_kv.get_length(batch_idx);
}
};
} // namespace flashinfer
#endif // FLASHINFER_DECODE_PARAMS_CUH_
|